package com.rjconsultores.ventaboletos.dao.hibernate;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.transform.AliasToBeanResultTransformer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;

import com.rjconsultores.ventaboletos.dao.ConexionDAO;
import com.rjconsultores.ventaboletos.entidad.Conexion;
import com.rjconsultores.ventaboletos.entidad.ConexionCtrlTemp;
import com.rjconsultores.ventaboletos.entidad.ConexionTemp;
import com.rjconsultores.ventaboletos.vo.conexion.ConexionVO;

@Repository("conexionDAO")
public class ConexionHibernateDAO extends GenericHibernateDAO<Conexion, Integer> implements ConexionDAO {
	private static Logger log = Logger.getLogger(ConexionHibernateDAO.class);

	@Autowired
	public ConexionHibernateDAO(@Qualifier("sessionFactory") SessionFactory factory) {
		setSessionFactory(factory);
	}

	@Override
	public List<ConexionVO> buscarConexiones(Integer origenId, Integer destinoId) {
		StringBuilder sb = new StringBuilder("");
		sb.append("select po1.descparada as conOrigen, pd1.descparada as conDestino ,c.numgrupo as grupo ,po2.descparada as tramoOrigen, pd2.descparada as tramoDestino ");
		sb.append("from ");
		sb.append("	ConexionCtrl cc, ");
		sb.append("	Conexion c, ");
		sb.append("	Parada po1, ");
		sb.append("	Parada pd1, ");
		sb.append("	Parada po2, ");
		sb.append("	Parada pd2 ");
		sb.append("where ");
		sb.append("	cc.conexionctrlId = c.conexionctrlId ");
		sb.append("	and po1.paradaId = cc.origenId ");
		sb.append("	and pd1.paradaId = cc.destinoId ");
		sb.append("	and po2.paradaId = c.origenId ");
		sb.append("	and pd2.paradaId = c.destinoId ");
		if (origenId != null){
			sb.append("	and cc.origenId =  ").append(origenId);	
		}
		if (destinoId != null){
			sb.append("	and cc.destinoId =  ").append(destinoId);	
		}
		sb.append("	 ");
		sb.append("	 ");
		sb.append("order by po1.descparada, pd1.descparada ,c.numgrupo, po2.descparada, pd2.descparada ");

		Query query = getSession().createQuery(sb.toString());
		query.setResultTransformer(new AliasToBeanResultTransformer(ConexionVO.class));

		return query.list();

	}

	@Override
	public void generarConexiones(Integer usuarioId) {
		this.getSession().createQuery("DELETE ConexionTemp").executeUpdate();
		this.getSession().createQuery("DELETE ConexionCtrlTemp").executeUpdate();

		Query queryParadas = this.getSession().createQuery("select distinct t.origem.paradaId,t.destino.paradaId from Tramo t where t.activo = 1 order by t.origem.paradaId ");
		List<Object[]> lsParadas = queryParadas.list();

		StringBuilder sb = new StringBuilder("");
		sb.append("SELECT ");
		sb.append("	t.origem.paradaId, ");
		sb.append("	t.destino.paradaId, ");
		sb.append("	t2.origem.paradaId, ");
		sb.append("	t2.destino.paradaId, ");
		sb.append("    MAX(t.kmReal), ");
		sb.append("    MAX(tcs.tiemporecorrido), ");
		sb.append("    MAX(t2.kmReal), ");
		sb.append("    MAX(tcs2.tiemporecorrido) ");
		sb.append(" ");
		sb.append("FROM ");
		sb.append("    Tramo t, ");
		sb.append("    Tramo t2, ");
		sb.append("    TramoServicio tcs, ");
		sb.append("    TramoServicio tcs2 ");
		sb.append("     ");
		sb.append("where ");
		sb.append("	t.origem.paradaId = :origen ");
		sb.append("	and t.activo=1 ");
		sb.append("	 ");
		sb.append("	and t2.destino.paradaId = :destino ");
		sb.append("	and t2.activo=1 ");
		sb.append("	and t2.origem= t.destino ");
		sb.append("	 ");
		sb.append("	and tcs.tramo=t ");
		sb.append("	and tcs.activo=1 ");
		sb.append("  ");
		sb.append("	and tcs2.tramo=t2 ");
		sb.append("	and tcs2.activo=1 ");
		sb.append("  ");
		sb.append("group by ");
		sb.append("    t.origem.paradaId, ");
		sb.append("    t.destino.paradaId, ");
		sb.append("    t2.origem.paradaId, ");
		sb.append("    t2.destino.paradaId ");

		Map<String, Long> mapConexionCtrlId = new HashMap<String, Long>();
		int grupo = 0;
		for (Object[] obj : lsParadas) {
			Integer origenId = (Integer) obj[0];
			Integer destinoId = (Integer) obj[1];

			Query query = getSession().createQuery(sb.toString());
			query.setParameter("origen", origenId);
			query.setParameter("destino", destinoId);

			List<Object[]> list = query.list();

			for (Object[] objConexion : list) {
				Integer origen1 = (Integer) objConexion[0];
				Integer destino1 = (Integer) objConexion[1];
				Integer origen2 = (Integer) objConexion[2];
				Integer destino2 = (Integer) objConexion[3];

				String claveConexionCtrl = origenId + "-" + destinoId;

				Long conexionCtrlId = mapConexionCtrlId.get(claveConexionCtrl);

				if (conexionCtrlId == null) {
					ConexionCtrlTemp c = new ConexionCtrlTemp();
					c.setActivo(true);
					c.setOrigenId(origen1);
					c.setDestinoId(destino2);
					c.setFecmodif(new java.util.Date());
					c.setUsuarioId(usuarioId);

					conexionCtrlId = (Long) this.getSession().save(c);
					mapConexionCtrlId.put(claveConexionCtrl, conexionCtrlId);

				}
				grupo++;

				ConexionTemp cTemp = new ConexionTemp();
				cTemp.setActivo(true);
				cTemp.setConexionctrlId(conexionCtrlId);
				cTemp.setOrigenId(origen1);
				cTemp.setDestinoId(destino1);
				cTemp.setNumgrupo(grupo);
				cTemp.setNumsecuencia((short) 1);
				cTemp.setUsuarioId(usuarioId);
				cTemp.setFecmodif(new java.util.Date());

				ConexionTemp cTemp2 = new ConexionTemp();
				cTemp2.setActivo(true);
				cTemp2.setConexionctrlId(conexionCtrlId);
				cTemp2.setOrigenId(origen2);
				cTemp2.setDestinoId(destino2);
				cTemp2.setNumgrupo(grupo);
				cTemp2.setNumsecuencia((short) 2);
				cTemp2.setUsuarioId(usuarioId);
				cTemp2.setFecmodif(new java.util.Date());

				getSession().save(cTemp);
				getSession().save(cTemp2);
			}
		}

		this.getSession().createQuery("DELETE Conexion").executeUpdate();
		this.getSession().createQuery("DELETE ConexionCtrl").executeUpdate();

		sb = new StringBuilder("");
		sb.append("INSERT INTO ConexionCtrl (conexionctrlId,fecmodif,activo,usuarioId,origenId,destinoId) ");
		sb.append("SELECT conexionctrlId,fecmodif,activo,usuarioId,origenId,destinoId  from ConexionCtrlTemp )");
		this.getSession().createQuery(sb.toString()).executeUpdate();

		sb = new StringBuilder("");
		sb.append("INSERT INTO Conexion (conexionId,numgrupo,numsecuencia,activo,fecmodif,usuarioId,origenId,destinoId,conexionctrlId) ");
		sb.append("SELECT conexionId,numgrupo,numsecuencia,activo,fecmodif,usuarioId,origenId,destinoId,conexionctrlId  from ConexionTemp )");
		this.getSession().createQuery(sb.toString()).executeUpdate();
	}
}
