/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.rjconsultores.ventaboletos.service;

import com.rjconsultores.ventaboletos.entidad.ClaseServicio;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.Parada;
import com.rjconsultores.ventaboletos.entidad.ParadaSecuencia;
import com.rjconsultores.ventaboletos.entidad.Ruta;
import com.rjconsultores.ventaboletos.exception.BusinessException;
import com.rjconsultores.ventaboletos.utilerias.RegistroConDependenciaException;
import java.util.List;

/**
 * 
 * @author Rafius
 */
public interface RutaService{

	/**
	 * Regresa el destino de la ruta
	 * 
	 * @param ruta
	 * @return
	 */
	public Parada getDestino(Ruta ruta);

	/**
	 * Regresa la origen de la ruta
	 * 
	 * @param ruta
	 * @return
	 */
	public Parada getOrigem(Ruta ruta);

	public List<Ruta> buscarPorRutaPorEmpresa(Ruta ruta, Empresa empresa);

	public List<Ruta> buscarPorClaseServicio(ClaseServicio claseServicio);

	public List<Ruta> buscar(String nomeRuta, ClaseServicio claseServicio, Boolean nomeObrigatorio);

	public List<Ruta> obtenerPorEmpresa(Empresa empresa);

	public List<Ruta> buscarTodosExceto(Integer... idRuta);

	public void borrar(Ruta entidad) throws RegistroConDependenciaException;

	/**
	 * Verifica se la parada informadad existe en la ruta informada
	 * 
	 * @param parada
	 * @param ruta
	 * @return
	 */
	public boolean paradaExisteEnLaRuta(Parada parada, Ruta ruta);

	/**
	 * Hace la actualizacin de la ruta.
	 * 
	 * Si el parametro <code>lsParadasSequencia</code> no es null, quieres decir que hube cambio en la secuencia y la nueva secuencia es la que esta en <code>lsParadasSequencia</code>.<br/>
	 * 
	 * Si el parametro <code>lsParadasSequencia</code> es null, no va a cambiar ni la secuencia ni las combinaciones<br/>
	 * @param ruta
	 * @param lsParadasSequencia
	 * @return
	 * @throws BusinessException
	 */
	public Ruta actualizacion(Ruta ruta, List<ParadaSecuencia> lsParadasSequencia) throws BusinessException;

	public List<Ruta> obtenerTodos();

	public Ruta obtenerID(Integer id);

	public Ruta suscribir(Ruta entidad, List<ParadaSecuencia> lsParadasSequencia) throws BusinessException;
	
	public void generarCombinacion(Ruta ruta) throws BusinessException ;
}
