/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.rjconsultores.ventaboletos.service.impl;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.CorridaDAO;
import com.rjconsultores.ventaboletos.dao.EsquemaCorridaDAO;
import com.rjconsultores.ventaboletos.dao.RutaDAO;
import com.rjconsultores.ventaboletos.entidad.ClaseServicio;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.Parada;
import com.rjconsultores.ventaboletos.entidad.ParadaSecuencia;
import com.rjconsultores.ventaboletos.entidad.Ruta;
import com.rjconsultores.ventaboletos.entidad.RutaCombinacion;
import com.rjconsultores.ventaboletos.entidad.RutaEmpresa;
import com.rjconsultores.ventaboletos.entidad.RutaSecuencia;
import com.rjconsultores.ventaboletos.entidad.Tramo;
import com.rjconsultores.ventaboletos.entidad.Via;
import com.rjconsultores.ventaboletos.exception.BusinessException;
import com.rjconsultores.ventaboletos.service.RutaCombinacionService;
import com.rjconsultores.ventaboletos.service.RutaSecuenciaService;
import com.rjconsultores.ventaboletos.service.RutaService;
import com.rjconsultores.ventaboletos.service.TramoService;
import com.rjconsultores.ventaboletos.utilerias.RegistroConDependenciaException;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

/**
 * 
 * @author Rafius
 */
@Service("rutaService")
public class RutaServiceImpl implements RutaService {

	@Autowired
	private RutaDAO rutaDAO;
	@Autowired
	private TramoService tramoService;
	@Autowired
	private RutaCombinacionService rutaCombinacionService;
	@Autowired
	private RutaSecuenciaService rutaSecuenciaService;
	@Autowired
	private CorridaDAO corridaDAO;
	@Autowired
	private EsquemaCorridaDAO esquemaCorridaDAO;

	private static Logger log = Logger.getLogger(RutaServiceImpl.class);

	public List<Ruta> obtenerTodos() {
		return rutaDAO.obtenerTodos();
	}

	public Ruta obtenerID(Integer id) {
		return rutaDAO.obtenerID(id);
	}

	@Transactional(rollbackFor=BusinessException.class)
	public Ruta suscribir(Ruta entidad, List<ParadaSecuencia> lsParadasSequencia) throws BusinessException {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		entidad = rutaDAO.suscribir(entidad);
		if (lsParadasSequencia != null) {
			generarSecuencias(entidad, lsParadasSequencia);
			generarCombinacion(entidad);
		}

		return entidad;
	}

	private void generarSecuencias(Ruta ruta, List<ParadaSecuencia> lsParadasSequencia) throws BusinessException {

		if (lsParadasSequencia != null) {
			List<RutaSecuencia> lsRutaSecuencia = new ArrayList<RutaSecuencia>();

			// Desativo todas as secuencias e gero novamente.
			rutaSecuenciaService.borrarSecuencias(ruta);

			log.info("Gerao de Secuencia!");
			log.info("Ruta: " + ruta.getRutaId() + " - " + ruta.getDescruta());

			lsRutaSecuencia.clear();
			StringBuilder sb = new StringBuilder("Tramo(s) inexistente(s):");
			sb.append("\n");
			boolean msg = false;
			for (int i = 0; i < lsParadasSequencia.size() - 1; i++) {
				Parada ori = lsParadasSequencia.get(i).getParada();
				Via via = lsParadasSequencia.get(i).getVia();
				Integer sec1 = lsParadasSequencia.get(i).getSecuencia().intValue();
				log.info("Secuencia: " + sec1 + " origen " + ori.getDescparada() + " - " + ori.getCveparada() + " Via: " + via.getNombvia());

				Parada des = lsParadasSequencia.get(i + 1).getParada();
				Integer sec2 = lsParadasSequencia.get(i + 1).getSecuencia().intValue();
				log.info("Secuencia: " + sec2 + " destino " + des.getDescparada() + " - " + des.getCveparada() + " Via: " + via.getNombvia());

				ClaseServicio claseServicio = ruta.getClaseServicio();

				Tramo tramo = tramoService.obtenerTramoUnicoRuta(ori, des, via, claseServicio);

				if (tramo != null) {

					RutaSecuencia rutaSecuencia = new RutaSecuencia();
					rutaSecuencia.setRuta(ruta);
					rutaSecuencia.setTramo(tramo);
					rutaSecuencia.setNumsecuencia(sec1.shortValue());
					rutaSecuencia.setActivo(Boolean.TRUE);
					rutaSecuencia.setFecmodif(Calendar.getInstance().getTime());
					rutaSecuencia.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());

					rutaSecuencia = rutaSecuenciaService.suscribir(rutaSecuencia);
					lsRutaSecuencia.add(rutaSecuencia);

				} else {
					throw new BusinessException("rutaServiceImpl.msg.validacionTRamo", new Object[] { ori.getCveparada(), des.getCveparada(), via.getNombvia(), claseServicio.getDescclase() });
				}
			}
		}
	}

	@Transactional(rollbackFor=BusinessException.class)
	public Ruta actualizacion(Ruta ruta, List<ParadaSecuencia> lsParadasSequencia) throws BusinessException {
		if (lsParadasSequencia != null) {
			generarSecuencias(ruta, lsParadasSequencia);
			generarCombinacion(ruta);
		}

		ruta.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		ruta.setFecmodif(Calendar.getInstance().getTime());
		ruta.setActivo(Boolean.TRUE);

		return rutaDAO.actualizacion(ruta);
	}

	@Transactional(rollbackFor=RegistroConDependenciaException.class)
	public void borrar(Ruta entidad) throws RegistroConDependenciaException {
		if ((corridaDAO.count(entidad) > 0l) || (esquemaCorridaDAO.count("ruta", entidad) > 0l)) {
			throw new RegistroConDependenciaException();
		}

		entidad = this.obtenerID(entidad.getRutaId());
		for (RutaCombinacion rutaCombinacion : entidad.getRutaCombinacionList()) {
			rutaCombinacion.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
			rutaCombinacion.setFecmodif(Calendar.getInstance().getTime());
			rutaCombinacion.setActivo(Boolean.FALSE);
		}

		for (RutaSecuencia rutaSecuencia : entidad.getRutaSecuenciaList()) {
			rutaSecuencia.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
			rutaSecuencia.setFecmodif(Calendar.getInstance().getTime());
			rutaSecuencia.setActivo(Boolean.FALSE);
		}

		for (RutaEmpresa rutaEmpresa : entidad.getLsRutaEmpresa()) {
			rutaEmpresa.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
			rutaEmpresa.setFecmodif(Calendar.getInstance().getTime());
			rutaEmpresa.setActivo(Boolean.FALSE);
		}

		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		rutaDAO.actualizacion(entidad);
	}

	@Transactional(rollbackFor=BusinessException.class)
	public void generarCombinacion(Ruta ruta) throws BusinessException {

		rutaCombinacionService.borrarCombinaciones(ruta);

		List<RutaCombinacion> listRutaCombinacion = new ArrayList<RutaCombinacion>();
		List<RutaSecuencia> lsRutaSecuencia = new ArrayList<RutaSecuencia>();

		lsRutaSecuencia = rutaSecuenciaService.buscarSecuenciaOrdenado(ruta);

		StringBuilder sb = new StringBuilder();
		Boolean gerou = true;

		for (int i = 0; i < lsRutaSecuencia.size(); i++) {
			Tramo tramoSecuencia1 = lsRutaSecuencia.get(i).getTramo();
			Parada origem = tramoSecuencia1.getOrigem();

			for (int x = 0; x < lsRutaSecuencia.size(); x++) {
				Tramo tramoSecuencia2 = lsRutaSecuencia.get(x).getTramo();
				Parada destinoOrigem = tramoSecuencia2.getDestino();

				// Numero secuencia anterior > Numero secuencia atual não deve fazer.
				// Essa condio não permiti gerar uma volta.
				if (lsRutaSecuencia.get(i).getNumsecuencia().intValue() > lsRutaSecuencia.get(x).getNumsecuencia().intValue()) {
					continue;
				}

				// IDA - A x B - A x C
				if (origem == destinoOrigem) {
					continue;
				}

				List<Tramo> listTramo = tramoService.obtenerPorOrigemDestinoFetchLazy(origem, destinoOrigem);

				if (listTramo.isEmpty()) {
					throw new BusinessException("rutaServiceImpl.msg.validacionTRamoCombinacion", new Object[] { origem.getDescparada(), origem.getCveparada(), destinoOrigem.getDescparada(), destinoOrigem.getCveparada() });
				}

				Tramo trBuscaOrigemDestino = new Tramo();
				for (Tramo tm : listTramo) {
					if (tm.getVia().equals(tramoSecuencia2.getVia())) {
						trBuscaOrigemDestino = tm;
						break;
					} else if (tm.getVia().equals(tramoSecuencia1.getVia())) {
						trBuscaOrigemDestino = tm;
						break;
					} else {
						trBuscaOrigemDestino = tm;
					}
				}
				
				log.debug("Tramo:"+trBuscaOrigemDestino.getDesctramo());

				RutaCombinacion rc = new RutaCombinacion();
				rc.setActivo(Boolean.TRUE);
				rc.setFecmodif(Calendar.getInstance().getTime());
				rc.setIndventa(Boolean.TRUE);
				rc.setRuta(ruta);
				rc.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
				rc.setTramo(trBuscaOrigemDestino);

				// Verifica se j existe o registro INATIVO para Reativa-lo e manter a configurao se vende.
				// if (!rutaCombinacionService.busquedaRutaTramo(ruta, trBuscaOrigemDestino, Boolean.FALSE)) {
				rc = rutaCombinacionService.suscribir(rc);
				// } else {
				// rutaCombinacionService.activar(ruta, trBuscaOrigemDestino);
				// }

			}
		}
	}

	public Parada getDestino(Ruta ruta) {
		return rutaDAO.buscarDestino(ruta);
	}

	public Parada getOrigem(Ruta ruta) {
		return rutaDAO.buscarOrigen(ruta);
	}

	public List<Ruta> buscarPorRutaPorEmpresa(Ruta ruta, Empresa empresa) {
		return rutaDAO.buscarPorRutaPorEmpresa(ruta, empresa);
	}

	public List<Ruta> buscarPorClaseServicio(ClaseServicio claseServicio) {
		return rutaDAO.buscarPorClaseServicio(claseServicio);
	}

	public List<Ruta> buscar(String nomeRuta, ClaseServicio claseServicio, Boolean nomeObrigatorio) {
		return rutaDAO.buscar(nomeRuta, claseServicio, nomeObrigatorio);
	}

	public List<Ruta> obtenerPorEmpresa(Empresa empresa) {
		return rutaDAO.obtenerPorEmpresa(empresa);
	}

	public List<Ruta> buscarTodosExceto(Integer... idRuta) {
		return rutaDAO.buscarTodosExceto(idRuta);
	}

	@Override
	public boolean paradaExisteEnLaRuta(Parada parada, Ruta ruta) {
		ruta = obtenerID(ruta.getRutaId());
		boolean existe = false;
		for (int i = 0; (i < ruta.getRutaSecuenciaList().size()) && !existe; i++) {
			RutaSecuencia rutaSecuencia = ruta.getRutaSecuenciaList().get(i);
			Tramo tramo = rutaSecuencia.getTramo();
			if (tramo != null) {
				if (tramo.getOrigem().equals(parada) || tramo.getDestino().equals(parada)) {
					existe = true;
				}
			}
		}
		return existe;
	}
}
