/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.rjconsultores.ventaboletos.utilerias;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.logging.Level;
import java.util.logging.Logger;

public final class DateUtil {

	/**
	 * No need for an instance
	 */
	private DateUtil() {
	}

	/**
	 * Elapsed days based on current time
	 * 
	 * @param date
	 *            Date
	 * 
	 * @return int number of days
	 */
	public static int getElapsedDias(Date date) {
		return elapsed(date, Calendar.DATE);
	}

	/**
	 * Elapsed days based on two Date objects
	 * 
	 * @param d1
	 *            Date
	 * @param d2
	 *            Date
	 * 
	 * @return int number of days
	 */
	public static int getElapsedDias(Date d1, Date d2) {
		return elapsed(d1, d2, Calendar.DATE);
	}

	/**
	 * Elapsed months based on current time
	 * 
	 * @param date
	 *            Date
	 * 
	 * @return int number of months
	 */
	public static int getElapsedMeses(Date date) {
		return elapsed(date, Calendar.MONTH);
	}

	/**
	 * Elapsed months based on two Date objects
	 * 
	 * @param d1
	 *            Date
	 * @param d2
	 *            Date
	 * 
	 * @return int number of months
	 */
	public static int getElapsedMeses(Date d1, Date d2) {
		return elapsed(d1, d2, Calendar.MONTH);
	}

	/**
	 * Elapsed years based on current time
	 * 
	 * @param date
	 *            Date
	 * 
	 * @return int number of years
	 */
	public static int getElapsedAnos(Date date) {
		return elapsed(date, Calendar.YEAR);
	}

	/**
	 * Elapsed years based on two Date objects
	 * 
	 * @param d1
	 *            Date
	 * @param d2
	 *            Date
	 * 
	 * @return int number of years
	 */
	public static int getElapsedAnos(Date d1, Date d2) {
		return elapsed(d1, d2, Calendar.YEAR);
	}

	/**
	 * All elaspsed types
	 * 
	 * @param g1
	 *            GregorianCalendar
	 * @param g2
	 *            GregorianCalendar
	 * @param type
	 *            int (Calendar.FIELD_NAME)
	 * 
	 * @return int number of elapsed "type"
	 */
	private static int elapsed(GregorianCalendar g1, GregorianCalendar g2, int type) {
		GregorianCalendar gc1, gc2;
		int elapsed = 0;
		// Create copies since we will be clearing/adding
		if (g2.after(g1)) {
			gc2 = (GregorianCalendar) g2.clone();
			gc1 = (GregorianCalendar) g1.clone();
		} else {
			gc2 = (GregorianCalendar) g1.clone();
			gc1 = (GregorianCalendar) g2.clone();
		}
		if (type == Calendar.MONTH || type == Calendar.YEAR) {
			gc1.clear(Calendar.DATE);
			gc2.clear(Calendar.DATE);
		}
		if (type == Calendar.YEAR) {
			gc1.clear(Calendar.MONTH);
			gc2.clear(Calendar.MONTH);
		}
		while (gc1.before(gc2)) {
			gc1.add(type, 1);
			elapsed++;
		}
		return elapsed;
	}

	/**
	 * All elaspsed types based on date and current Date
	 * 
	 * @param date
	 *            Date
	 * @param type
	 *            int (Calendar.FIELD_NAME)
	 * 
	 * @return int number of elapsed "type"
	 */
	private static int elapsed(Date date, int type) {
		return elapsed(date, new Date(), type);
	}

	/**
	 * All elaspsed types
	 * 
	 * @param d1
	 *            Date
	 * @param d2
	 *            Date
	 * @param type
	 *            int (Calendar.FIELD_NAME)
	 * 
	 * @return int number of elapsed "type"
	 */
	private static int elapsed(Date d1, Date d2, int type) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(d1);
		GregorianCalendar g1 = new GregorianCalendar(cal.get(Calendar.YEAR), cal.get(Calendar.MONTH), cal.get(Calendar.DATE));
		cal.setTime(d2);
		GregorianCalendar g2 = new GregorianCalendar(cal.get(Calendar.YEAR), cal.get(Calendar.MONTH), cal.get(Calendar.DATE));
		return elapsed(g1, g2, type);
	}

	public static Date getDateFromString(String data, String formato) throws java.text.ParseException {
		Date d = new Date();
		DateFormat df = new SimpleDateFormat(formato);

		d = df.parse(data);
		return d;
	}

	public static int compareDate(java.util.Date d1) {
		return compareDate(d1, new java.util.Date());
	}

	public static int compareOnlyDate(java.util.Date d1) {
		return compareOnlyDate(d1, new java.util.Date());
	}

	/**
	 * Não leva em consideração os segundos
	 */
	public static int compareOnlyDate(java.util.Date d1, java.util.Date d2) {
		java.util.Calendar cal = Calendar.getInstance();
		cal.setTime(d1);
		GregorianCalendar g1 = new GregorianCalendar(cal.get(Calendar.YEAR), cal.get(Calendar.MONTH), cal.get(Calendar.DATE));

		java.util.Calendar cal2 = Calendar.getInstance();
		cal2.setTime(d2);
		GregorianCalendar g2 = new GregorianCalendar(cal2.get(Calendar.YEAR), cal2.get(Calendar.MONTH), cal2.get(Calendar.DATE));
		return g1.compareTo(g2);
		// return cal.compareTo(cal2);
	}

	/**
	 * Retorna : -  0 se igual </br>
	 * 			 -  < 0 se d1 < d2 </br> 
	 * 			 -  > 0 se d1 > d2 </br>
	 */
	public static int compareDate(java.util.Date d1, java.util.Date d2) {
		java.util.Calendar cal = Calendar.getInstance();
		cal.setTime(d1);
		java.util.Calendar cal2 = Calendar.getInstance();
		cal2.setTime(d2);

		return cal.compareTo(cal2);
	}

	public static Integer getIntegerDate(java.util.Date d, String formato) {
		if (formato.contains("/")) {
			throw new IllegalArgumentException("O formato não deve conter o simbolo \\ ");
		}

		String s = getStringDate(d, formato);
		return Integer.parseInt(s);
	}

	public static String getStringDate(java.util.Date d, String formato) {
		DateFormat df = new SimpleDateFormat(formato);
		return df.format(d);
	}

	public static String getStringDate(java.util.Date d) {
		return getStringDate(d, "dd/MM/yyyy");
	}

	public static String getStringDate(java.sql.Date d) {
		return getStringDate(new java.util.Date(d.getTime()));
	}

	public static String getStringCurrentDate(String formato) {
		return getStringDate(new java.util.Date(), formato);
	}

	public static String changeFormatStringDate(String date, String currentFormat, String newFormat) {

		try {
			return DateUtil.getStringDate(DateUtil.getDateFromString(date, currentFormat), newFormat);
		} catch (ParseException ex) {
			Logger.getLogger(DateUtil.class.getName()).log(Level.SEVERE, null, ex);
		}
		return null;
	}

	public static Integer changeFormatIntDate(Integer date, String currentFormat, String newFormat) {

		try {
			return DateUtil.getIntegerDate(DateUtil.getDateFromString(String.valueOf(date), currentFormat), newFormat);
		} catch (ParseException ex) {
			Logger.getLogger(DateUtil.class.getName()).log(Level.SEVERE, null, ex);
		}
		return null;
	}

	/**
	 * Retorna uma array onde [0] é hora, [1] é minuto
	 * 
	 * @param hora
	 * @return
	 */
	private static int[] getArrayHora(Integer hora) {

		String strHora = String.valueOf(hora);

		int[] array = new int[2];

		if (strHora.length() <= 2) {
			array[0] = 0;
			array[1] = Integer.parseInt(strHora);
		} else if (strHora.length() == 3) {
			array[0] = Integer.parseInt(strHora.substring(0, 1));
			array[1] = Integer.parseInt(strHora.substring(1));
		} else if (strHora.length() == 4) {
			array[0] = Integer.parseInt(strHora.substring(0, 2));
			array[1] = Integer.parseInt(strHora.substring(2));
		}

		return array;
	}

	public static java.util.Date changeFormatIntDate(Integer date, String currentFormat, Integer hora) {

		try {
			if (currentFormat.equals("yMMdd") || currentFormat.equals("yyMMdd")) {

				currentFormat = "yyyyMMdd";
				String strData = String.valueOf(date);

				if (strData.length() == 5) {
					strData = "200" + strData;
				} else if (strData.length() == 6) {
					strData = "20" + strData;
				}

				date = Integer.parseInt(strData);
			}
			Date data = DateUtil.getDateFromString(String.valueOf(date), currentFormat);

			if (hora == null) {
				return data;
			}

			int horaArray[] = DateUtil.getArrayHora(hora);
			Calendar c = Calendar.getInstance();
			c.setTime(data);
			c.set(Calendar.HOUR_OF_DAY, horaArray[0]);
			c.set(Calendar.MINUTE, horaArray[1]);

			return c.getTime();

		} catch (ParseException ex) {
			Logger.getLogger(DateUtil.class.getName()).log(Level.SEVERE, null, ex);
		}
		return null;
	}

	public static java.util.Date changeFormatStringDate(String date, String currentFormat, Integer hora) {

		try {
			if (currentFormat.equals("yMMdd") || currentFormat.equals("yyMMdd")) {

				currentFormat = "yyyyMMdd";

				if (date.length() == 5) {
					date = "200" + date;
				} else if (date.length() == 6) {
					date = "20" + date;
				}
			}
			Date data = DateUtil.getDateFromString(String.valueOf(date), currentFormat);

			if (hora == null) {
				return data;
			}

			int horaArray[] = DateUtil.getArrayHora(hora);
			Calendar c = Calendar.getInstance();
			c.setTime(data);
			c.set(Calendar.HOUR_OF_DAY, horaArray[0]);
			c.set(Calendar.MINUTE, horaArray[1]);

			return c.getTime();

		} catch (ParseException ex) {
			Logger.getLogger(DateUtil.class.getName()).log(Level.SEVERE, null, ex);
		}
		return null;
	}

	public static int getIntHoraSRVP(Date data) {
		Calendar c = Calendar.getInstance();
		c.setTime(data);

		String hora = String.valueOf(c.get(Calendar.HOUR_OF_DAY));
		hora = (hora.length() < 2) ? "0" + hora : hora;

		String minuto = String.valueOf(c.get(Calendar.MINUTE));
		minuto = (minuto.length() < 2) ? "0" + minuto : minuto;

		return Integer.parseInt(hora + minuto);
	}

	public static long getElapsedHoras(Date dtInic, Date dtFim) {
		return (dtFim.getTime() - dtInic.getTime()) / (1000 * 60 * 60);
	}

	public static long getElapsedMinutos(Date dtInic, Date dtFim) {
		return (dtFim.getTime() - dtInic.getTime()) / (1000 * 60);
	}

	/**
	 * Regresa los minutos pela hora inicial del sistema
	 * 
	 * @param dtInic
	 * @param dtFim
	 * @return
	 */
	public static Hora getHorasMinutosByInicio(Date dtFim) {
		return new Hora(dtFim);
	}

	/**
	 * Verifica se o intervalo de datas dtStartCompare e dtEndCompare estão
	 * dentro do intervalo dtStart dtEnd
	 * 
	 * @param dtStart
	 * @param dtEnd
	 * @param dtStartCompare
	 * @param dtEndCompare
	 * @return
	 */
	public static boolean intersectDate(Date dtStart, Date dtEnd, Date dtStartCompare, Date dtEndCompare) {
		return TraslaparUtil.intersectDate(dtStart, dtEnd, dtStartCompare, dtEndCompare);
	}

	public static void main(String ar[]) {
		// System.out.println(DateUtil.intersectNum(0, 100, 50, 150));
	}

	public static void main2(String args[]) {
		java.util.Date agora = new java.util.Date();

		System.out.println(agora);

		Calendar c = Calendar.getInstance();
		c.setTime(agora);

		System.out.println(DateUtil.getIntHoraSRVP(agora));
		// c.add(Calendar.MINUTE, 60);

		System.out.println("final " + c.getTime());
		System.out.println("1 " + c.get(Calendar.HOUR_OF_DAY));

	}

	/**
	 * Es la fecInicial del sistema para campos de horas
	 * 
	 * @return
	 */
	public static Calendar getFecInicio() {
		Calendar c = Calendar.getInstance();
		c.set(Calendar.YEAR, 1970);
		c.set(Calendar.DAY_OF_MONTH, 1);
		c.set(Calendar.MONTH, 0);
		c.set(Calendar.HOUR_OF_DAY, 0);
		c.set(Calendar.MINUTE, 0);
		c.set(Calendar.MILLISECOND, 0);
		c.set(Calendar.SECOND, 0);

		return c;
	}
	/**
	 * Es la fecInicial del sistema con los campos de hora y minutos del parametro informado
	 * 
	 * @return
	 */
	public static Calendar getFecInicio(Date horaMinuto) {
		Calendar tmp = Calendar.getInstance();
		tmp.setTime(horaMinuto);
		Calendar c = getFecInicio();
		c.set(Calendar.HOUR_OF_DAY, tmp.get(Calendar.HOUR_OF_DAY));
		c.set(Calendar.MINUTE, tmp.get(Calendar.MINUTE));
		
		
		return c;
	}
	/**
	 * Es la fecInicial del sistema con los campos de hora y minutos del parametro informado
	 * 
	 * @return
	 */
	public static Calendar getFecInicio(Integer hora,Integer minuto) {
		
		Calendar c = getFecInicio();
		c.add(Calendar.HOUR_OF_DAY, hora);
		c.add(Calendar.MINUTE, minuto);
		
		
		return c;
	}

	/**
	 * Quita los segundos y milesegundos de la fecha
	 * 
	 * @param fecha
	 * @return
	 */
	public static Date normalizar(Date fecha) {
		GregorianCalendar gcalendar = new GregorianCalendar();
		gcalendar.setTime(fecha);
		gcalendar.set(Calendar.SECOND, 0);
		gcalendar.set(Calendar.MILLISECOND, 0);

		return gcalendar.getTime();
	}
	/**
	 * Quita los segundos, milesegundos, hora y minuto de la fecha
	 * 
	 * @param fecha
	 * @return
	 */
	public static Date normalizarToFecha(Date fecha) {
		GregorianCalendar gcalendar = new GregorianCalendar();
		gcalendar.setTime(fecha);
		gcalendar.set(Calendar.SECOND, 0);
		gcalendar.set(Calendar.MILLISECOND, 0);
		gcalendar.set(Calendar.HOUR_OF_DAY, 0);
		gcalendar.set(Calendar.MINUTE, 0);
		
		return gcalendar.getTime();
	}

	public static class Hora {
		private Integer hora;
		private Integer minuto;

		public Hora() {
			this.hora = 0;
			this.minuto = 0;
		}

		public Hora(Date fecha) {
			conf(fecha);
		}

		private void conf(Date fecha) {
			Long tempo = DateUtil.getElapsedMinutos(DateUtil.getFecInicio().getTime(), fecha);

			long minutos = (tempo % 60);
			int horas = (tempo.intValue() / 60);

			hora = horas;
			minuto = (int) minutos;

		}

		@Override
		public String toString() {

			return (hora + ":" + ((String.valueOf(minuto).length() < 2) ? "0" + minuto : minuto));
		}

		public Integer getHora() {
			return hora;
		}

		public Integer getMinuto() {
			return minuto;
		}

		public void setMinuto(Integer minuto) {
			this.minuto = minuto;
		}

		public void setHora(Integer hora) {
			this.hora = hora;
		}

		public Date getFecha() {
			Calendar c = getFecInicio();
			c.add(Calendar.HOUR_OF_DAY, hora);
			c.add(Calendar.MINUTE, minuto);

			return c.getTime();
		}

		public Hora sumar(Hora horaAdicao) {
			Calendar horaAtual = Calendar.getInstance();
			horaAtual.setTime(getFecha());

			Calendar horaSomar = Calendar.getInstance();
			horaSomar.setTime(horaAdicao.getFecha());

			int horaTotal = horaAtual.get(Calendar.HOUR) + horaSomar.get(Calendar.HOUR);
			int minutoTotal = horaAtual.get(Calendar.MINUTE) + horaSomar.get(Calendar.MINUTE);

			Calendar fecInicio = getFecInicio();
			fecInicio.add(Calendar.HOUR_OF_DAY, horaTotal);
			fecInicio.add(Calendar.MINUTE, minutoTotal);

			return new Hora(fecInicio.getTime());
		}

	}
}